import os
from typing import Annotated

from fastapi import Depends
from sqlalchemy import create_engine
from sqlmodel import SQLModel, Session

from ..util.singleton import SingletonMeta


class DB(metaclass=SingletonMeta):

    def __init__(self):
        print(f"  DB - Initializing...")

        username = os.environ.get('POSTGRES_USER')
        password = os.environ.get('POSTGRES_PASSWORD')
        host = os.environ.get('POSTGRES_HOST')
        port = os.environ.get('POSTGRES_PORT')
        db = os.environ.get('POSTGRES_DB')

        self.sqlite_url = f"postgresql://{username}:{password}@{host}:{port}/{db}"

        self.engine = create_engine(self.sqlite_url)
        self.create_db_and_tables()

    def create_db_and_tables(self):
        SQLModel.metadata.create_all(self.engine)

    def get_session(self):
        with Session(self.engine) as session:
            yield session

    SessionDep = Annotated[Session, Depends(get_session)]



import os
from datetime import datetime
from typing import Optional
from uuid import UUID

from httpx import AsyncClient

from ..campaigns.models import ChannelType, CampaignGoal, Campaign as CampaignModel
from ..general.dataclasses import ResolvableCompanyModel
from ..integrations.dataclasses import Link


class Campaign(ResolvableCompanyModel):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]
    organization: UUID
    name: str
    external_id: Optional[str]
    status: str
    last_sync_date: datetime
    ad_count: int
    channel: Link
    channel_type: ChannelType
    campaign_goal: Optional[CampaignGoal]

    @staticmethod
    async def resolve_object(object_id: UUID, organization_id: UUID) -> "Campaign | None":
        result = await ResolveClient.campaigns().get(f"campaigns/{object_id}", headers={"X-Organization-ID": organization_id})

        if result.status_code != 200:
            return None

        return Campaign(**result.json())

    @classmethod
    async def from_model(cls: "Campaign", model: CampaignModel) -> 'Campaign':
        channel = await Link.resolve_object(model.channel_id, model.organization)

        return Campaign(
            id=model.id,
            created_at=model.created_at,
            updated_at=model.updated_at,
            organization=model.organization,
            name=model.name,
            external_id=model.external_id,
            status=model.status,
            last_sync_date=model.last_sync_date,
            ad_count=model.ad_count,
            channel=channel,
            channel_type=model.channel_type,
            campaign_goal=model.campaign_goal,
        )



import os
from datetime import datetime
from typing import List, Optional
from uuid import UUID
from dataclasses import field

from httpx import AsyncClient
from pydantic import BaseModel

from ..general.dataclasses import ResolvableCompanyModel
from ..integrations.dataclasses import Link
from ..util.enum import to_enum
from ..vacancies.models import Feed as FeedModel, Vacancy as VacancyModel, SalaryFrequency, RemoteType


class VacancyLocation(BaseModel):
    zip_code: str | None = None
    city: str | None = None
    address: str | None = None
    state: str | None = None
    country: str | None = None


class Salary(BaseModel):
    min: float | None = None
    max: float | None = None
    currency: str = "EUR"
    frequency: SalaryFrequency = "month"


class Hours(BaseModel):
    min: int = 0
    max: int = 40
    fte: float = 1.0


class ContactDetails(BaseModel):
    email: str | None = None
    first_name: str | None = None
    last_name: str | None = None
    phone_number: str | None = None
    role: str | None = None


class VacancyData(BaseModel):

    # Required fields
    reference_number: str
    requisition_id: str
    title: str
    description: str
    job_site_url: str
    company_name: str
    publish_date: datetime | None = None
    category: List[str] = field(default_factory=list)
    experience: List[str] = field(default_factory=list)
    education: List[str] = field(default_factory=list)

    # Connected data
    hours: Hours = field(default_factory=Hours)
    location: VacancyLocation = field(default_factory=VacancyLocation)
    salary: Salary = field(default_factory=Salary)
    recruiter: ContactDetails = field(default_factory=ContactDetails)

    # Optional fields
    status: str | None = None
    parent_company_name: str | None = None
    remote_type: RemoteType | None = None
    expiration_date: datetime | None = None
    last_updated_date: datetime | None = None

    def to_model(self, feed: FeedModel):
        return VacancyModel(
            feed_id=feed.id,
            organization=feed.organization,
            reference_number=self.reference_number,
            requisition_id=self.requisition_id,
            title=self.title,
            description=self.description,
            status=self.status,
            job_site_url=self.job_site_url,
            company_name=self.company_name,
            parent_company_name=self.parent_company_name,
            remote_type=self.remote_type.value if self.remote_type else None,
            publish_date=self.publish_date,
            expiration_date=self.expiration_date,
            last_updated_date=self.last_updated_date,
            category=self.category,
            experience=self.experience,
            education=self.education,
            hours_fte=self.hours.fte,
            hours_min=self.hours.min,
            hours_max=self.hours.max,
            location_address=self.location.address,
            location_zipcode=self.location.zip_code,
            location_city=self.location.city,
            location_state=self.location.state,
            location_country=self.location.country,
            salary_min=self.salary.min,
            salary_max=self.salary.max,
            salary_currency=self.salary.currency,
            salary_frequency=self.salary.frequency.value if self.salary.frequency else SalaryFrequency.MONTH,
            recruiter_first_name=self.recruiter.first_name,
            recruiter_last_name=self.recruiter.last_name,
            recruiter_phone_number=self.recruiter.phone_number,
            recruiter_email=self.recruiter.email,
            recruiter_role=self.recruiter.role,
        )


class Vacancy(VacancyData):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]
    organization: UUID
    feed_id: UUID

    @classmethod
    def from_model(cls: "Vacancy", model: VacancyModel) -> "Vacancy":
        hours = Hours(
            min=model.hours_min,
            max=model.hours_max,
            fte=model.hours_fte,
        )

        location = VacancyLocation(
            address=getattr(model, "location_address", None),
            zip_code=getattr(model, "location_zipcode", None),
            city=getattr(model, "location_city", None),
            state=getattr(model, "location_state", None),
            country=getattr(model, "location_country", None),
        )

        salary = Salary(
            min=getattr(model, "salary_min", None),
            max=getattr(model, "salary_max", None),
            currency=getattr(model, "salary_currency", "EUR") or "EUR",
            frequency=to_enum(SalaryFrequency, getattr(model, "salary_frequency", SalaryFrequency.MONTH) or SalaryFrequency.MONTH),
        )

        recruiter = ContactDetails(
            first_name=getattr(model, "recruiter_first_name", None),
            last_name=getattr(model, "recruiter_last_name", None),
            phone_number=getattr(model, "recruiter_phone_number", None),
            email=getattr(model, "recruiter_email", None),
            role=getattr(model, "recruiter_role", None),
        )

        remote_type = to_enum(RemoteType, getattr(model, "remote_type", None))

        return cls(
            organization=model.organization,
            id=model.id,
            created_at=model.created_at,
            updated_at=model.updated_at,
            feed_id=model.feed_id,

            reference_number=model.reference_number,
            requisition_id=model.requisition_id,
            title=model.title,
            description=model.description,
            job_site_url=model.job_site_url,
            company_name=model.company_name,
            publish_date=model.publish_date,
            category=list(model.category or []),
            experience=list(model.experience or []),
            education=list(model.education or []),

            hours=hours,
            location=location,
            salary=salary,
            recruiter=recruiter,

            status=model.status,
            parent_company_name=model.parent_company_name,
            remote_type=remote_type,
            expiration_date=model.expiration_date,
            last_updated_date=model.last_updated_date,
        )


class Feed(ResolvableCompanyModel):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]
    organization: UUID

    name: str
    status: str
    file_url: Optional[str]
    ats_link: Optional[Link]
    last_sync_date: Optional[datetime]
    synced_vacancy_count: int
    mapping: dict
    custom_fields: dict

    @staticmethod
    async def resolve_object(object_id: UUID, organization_id: UUID) -> "Feed | None":
        result = await ResolveClient.vacancies().get(f"feeds/{object_id}", headers={"X-Organization-ID": organization_id})

        if result.status_code != 200:
            return None

        return Feed(**result.json())

    @classmethod
    async def from_model(cls: "Feed", model: FeedModel, ats_link: Optional[Link] = None) -> "Feed":
        ats_link = await Link.resolve_object(model.ats_link_id, model.organization) if ats_link is None else ats_link

        return cls(
            id=model.id,
            created_at=model.created_at,
            updated_at=model.updated_at,
            organization=model.organization,
            name=model.name,
            status=model.status,
            file_url=model.file_url,
            ats_link=ats_link,
            last_sync_date=model.last_sync_date,
            synced_vacancy_count=model.synced_vacancy_count,
            mapping=model.mapping,
            custom_fields=model.custom_fields,
        )


from abc import abstractmethod, ABC
from uuid import UUID

from pydantic import BaseModel

class ResolvableModel(BaseModel, ABC):

    @staticmethod
    @abstractmethod
    async def resolve_object(object_id: UUID) -> "ResolvableModel":
        pass


class ResolvableCompanyModel(BaseModel, ABC):

    @staticmethod
    @abstractmethod
    async def resolve_object(object_id: UUID, organization_id: str) -> "ResolvableCompanyModel":
        pass


from uuid import UUID

from sqlmodel import Field

from ..general.models import IAMModel, IAMOrganizationModel
from sqlalchemy import Column, JSON


class CompositeRole(IAMModel, table=True):
    name: str = Field(index=True)
    roles: list = Field(sa_column=Column(JSON))
    organization: UUID = Field(index=True, nullable=True)
    manageable_roles: list = Field(sa_column=Column(JSON))
    permission_level: int = Field(default=0)


class MemberRoleConnection(IAMOrganizationModel, table=True):
    composite_role_id: UUID = Field(foreign_key="compositerole.id")
    member: UUID = Field(index=True, nullable=False)



from uuid import UUID
import enum

from datetime import datetime
from enum import StrEnum
from typing import Optional

from sqlalchemy import Column, JSON, Enum
from sqlmodel import Field

from ..general.models import CampaignsOrganizationModel


class ChannelType(StrEnum):
    JOB_BOARD = 'job-board'
    SOCIAL = 'social'


class CampaignGoal(enum.Enum):
    REACH = 'reach'
    TRAFFIC = 'traffic'
    CONVERSION = 'conversion'
    LEADS = 'leads'


class Campaign(CampaignsOrganizationModel, table=True):
    name: str = Field(index=True)
    external_id: Optional[str] = Field(index=True)
    status: str = Field(index=True)
    last_sync_date: Optional[datetime] = Field()
    ad_count: int = Field(default=0)
    channel_id: UUID = Field(index=True)
    channel_type: ChannelType = Field(sa_column=Column(Enum(ChannelType)))
    campaign_goal: Optional[CampaignGoal] = Field(sa_column=Column(Enum(CampaignGoal)))


class VacancySelection(CampaignsOrganizationModel):
    campaign_id: UUID = Field(foreign_key="campaign.id")
    feed_id: UUID = Field(index=True)
    selection_criteria: dict = Field(sa_column=Column(JSON))


class AdSet(CampaignsOrganizationModel, table=True):
    name: str = Field(index=True)
    external_id: Optional[str] = Field(index=True)
    campaign_id: UUID = Field(foreign_key="campaign.id")
    platforms: list = Field(sa_column=Column(JSON))
    ad_types: list = Field(sa_column=Column(JSON))
    settings: dict = Field(sa_column=Column(JSON))


class TargetLocation(CampaignsOrganizationModel, table=True):
    ad_set: UUID = Field(foreign_key="adset.id")
    address: str = Field(index=True)
    distance: int = Field(index=True)


class TargetAudience(CampaignsOrganizationModel, table=True):
    ad_set: UUID = Field(foreign_key="adset.id")
    age_min: int = Field(index=True, default=18)
    age_max: int = Field(index=True, default=150)
    interests: list = Field(sa_column=Column(JSON))
    languages: list = Field(sa_column=Column(JSON))


class Ad(CampaignsOrganizationModel, table=True):
    name: str = Field(index=True)
    external_id: Optional[str] = Field(index=True)
    campaign_id: UUID = Field(foreign_key="campaign.id")
    ad_set_id: Optional[UUID] = Field(foreign_key="adset.id")
    vacancy_id: Optional[UUID] = Field()
    lead_form: Optional[UUID] = Field(foreign_key="leadform.id")
    primary_text: str = Field()
    title: str = Field()
    description: Optional[str] = Field()
    conversion_goal: Optional[str] = Field()


class LeadForm(CampaignsOrganizationModel, table=True):
    title: str = Field()



import os
from uuid import UUID
from datetime import datetime
from typing import Optional

from httpx import AsyncClient

from .models import Link as LinkModel, Integration as IntegrationModel
from ..general.dataclasses import ResolvableModel, ResolvableCompanyModel
from ..resolve_clients import ResolveClient


class Integration(ResolvableModel):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]

    name: str
    icon: str
    type: str
    tag: Optional[str]
    enabled: bool
    description: str

    @staticmethod
    async def resolve_object(object_id: UUID) -> "Integration | None":
        result = await ResolveClient.integrations().get(f"integrations/{object_id}")

        if result.status_code != 200:
            return None

        return Integration(**result.json())

    @classmethod
    def from_model(cls: "Integration", model: IntegrationModel) -> "Integration":
        return cls(
            id=model.id,
            created_at=model.created_at,
            updated_at=model.updated_at,
            name=model.name,
            icon=model.icon,
            type=model.type,
            tag=model.tag,
            enabled=model.enabled,
            description=model.description,
        )


class Link(ResolvableCompanyModel):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]
    organization: UUID

    name: str
    status: str
    integration: Integration

    @staticmethod
    async def resolve_object(object_id: UUID, organization_id: UUID) -> "Link | None":
        result = await ResolveClient.integrations().get(f"links/{object_id}", headers={"X-Organization-ID": organization_id})

        if result.status_code != 200:
            return None

        return Link(**result.json())

    @classmethod
    async def from_model(cls: "Link", model: LinkModel) -> "Link | None":
        integration = await Integration.resolve_object(model.integration_id)

        return cls(
            id=model.id,
            created_at=model.created_at,
            updated_at=model.updated_at,
            organization=model.organization,
            name=model.name,
            status=model.status,
            integration=integration,
        )
